#!/usr/bin/env ruby

# == Synopsis
#
# nfn2dlp.rb: Script to transform an NFN (normal form nested) program
# into a disjunctive logic program
#
# == Usage
#
# nfn2dlp.rb [OPTION]
#
# -h, --help:
#    show help
#
# --outputfile file, -o file:
#    write the transformed program into file
#
# If no output file is specified, output will be written on standard output.


require 'rubygems'
require 'treetop'

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require 'nfn'
require 'rewriter_NFN_rule_class'

require 'getoptlong'
require 'rdoc/usage'

# Defaults for debug output and output filename.
debug=false
output_filename = nil

opts = GetoptLong.new(
                      [ '--help', '-h', GetoptLong::NO_ARGUMENT ],
                      [ '--outputfile', '-o', GetoptLong::REQUIRED_ARGUMENT ],
                      [ '--debug', GetoptLong::NO_ARGUMENT ]
    )

begin
  opts.each do |opt, arg|
    case opt
    when '--help'
      RDoc::usage
    when '--outputfile'
      output_filename = arg
    when '--debug'
      debug = true
    else
      RDoc::usage 255
    end
  end
rescue
      RDoc::usage 99
end  

if output_filename
  output_file = File.open(output_filename,"w")
else
  output_file = $stdout
end

# Create a treetop parser object for the grammar defined in
# nfn.treetop .
parser = NFNParser.new

errorcode = 0

# Number the rules.
rule_number = 0

ARGV.each do |input_filename|

  if ! File.exists?(input_filename)
    $stderr.print "Warning: File ", input_filename,
      " does not exist, skipped.\n"
    errorcode = 1
    next
  end

  if ! File.readable?(input_filename)
    $stderr.print "Warning: File ", input_filename,
      " is not readable, skipped.\n"
    errorcode = 1
    next
  end

  tree = parser.parse(File.read(input_filename))

  if tree
    tree.rules.each do |rule|
      # the DLP rule is rewritten iff it is safe
      if rule.isSafe == true
	
	# initialize object RewriterNFNrule
	rew_nfn = RewriterNFNrule.new(rule)
	# rewriteNFNrule rewrite the rule
	rew_nfn.rewriteNFNrule(rule_number)
	
	output_file.puts rew_nfn.std_rules_chr
      else
	$stderr.print "Rule ", rule, " is not safe.\n"
        errorcode = 2
	break
      end
      # increase rule count
      rule_number += 1
    end
  else
    $stderr.print "Parse error.\n"
    errorcode = 3
  end
end

if output_filename
  output_file.close
end


exit errorcode
