#!/usr/bin/env ruby

# == Synopsis
#
# nfnsolve.rb: Script to evaluate NFN (normal form nested) programs
#              using DLV.
#
# == Usage
#
# nfnsolve.rb [OPTION]
#
# -h, --help:
#    show help
#
# --dlv dlv-executable, -d dlv-executable:
#    use dlv-executable to compute answer sets
#
# --dlvoptions options:
#    pass additional options to dlv

require 'rubygems'
require 'treetop'

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require 'nfn'
require 'rewriter_NFN_rule_class'

require 'getoptlong'
require 'rdoc/usage'

# Defaults for debug output, dlv and its options.
debug=false
dlv="DLV"
dlvoptions=""

opts = GetoptLong.new(
                      [ '--help', '-h', GetoptLong::NO_ARGUMENT ],
                      [ '--dlv', '-d', GetoptLong::REQUIRED_ARGUMENT ],
                      [ '--dlvoptions', GetoptLong::REQUIRED_ARGUMENT ],
                      [ '--debug', GetoptLong::NO_ARGUMENT ]
    )

begin
  opts.each do |opt, arg|
    case opt
    when '--help'
      RDoc::usage
    when '--dlv'
      dlv = arg
    when '--dlvoptions'
      dlvoptions = arg
    when '--debug'
      debug = true
    else
      RDoc::usage 255
    end
  end
rescue
      RDoc::usage 99
end  

# Create a treetop parser object for the grammar defined in
# nfn.treetop .
parser = NFNParser.new

errorcode = 0

rewritten = String.new
predfilter = String.new

# Number the rules.
rule_number = 0

ARGV.each do |input_filename|

  if ! File.exists?(input_filename)
    $stderr.print "Warning: File ", input_filename,
      " does not exist, skipped.\n"
    errorcode = 1
    next
  end

  if ! File.readable?(input_filename)
    $stderr.print "Warning: File ", input_filename,
      " is not readable, skipped.\n"
    errorcode = 1
    next
  end

  tree = parser.parse(File.read(input_filename))

  if tree
    if debug
      print "filter: ", tree.predicates.keys.join(", "), "\n"
    end
    predfilter = tree.predicates.keys.join(",")

    tree.rules.each do |rule|
      # the DLP rule is rewritten iff it is safe
      if rule.isSafe == true
	
	# initialize object RewriterNFNrule
	rew_nfn = RewriterNFNrule.new(rule)
	# rewriteNFNrule rewrite the rule
	rew_nfn.rewriteNFNrule(rule_number)
	
	rewritten += rew_nfn.std_rules_chr.join("\n")
      else
	$stderr.print "Rule ", rule, " is not safe.\n"
        errorcode = 2
	break
      end
      # increase rule count
      rule_number += 1
    end
  else
    $stderr.print "Parse error.\n"
    errorcode = 3
  end
end

if debug
  $stdout.print rewritten
end

if errorcode == 0
  f = IO.popen(dlv + " " + dlvoptions + " --" + " -pfilter=" + predfilter,"w")
  f.puts rewritten
  f.close
end

exit errorcode
